<?php

namespace FiloBlu\CustomerBlocker\Observer\Sales;

class OrderPlaceBeforeAbstract implements \Magento\Framework\Event\ObserverInterface {

    const EMAIL_BLOCKER_ENABLE_CONFIG_PATH = 'filoblu_customer_blocker/block_by_email_address/enable';
    const EMAIL_BLOCKER_ADDRESSES_CONFIG_PATH = 'filoblu_customer_blocker/block_by_email_address/email_addresses_to_block';
    const CUSTOMER_BLOCKER_ADDRESSES_CONFIG_PATH = 'filoblu_customer_blocker/block_by_email_address/customer_addresses_to_block';

    protected $_scopeConfig;
    protected $messageManager;

    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Message\ManagerInterface $messageManager) {
        $this->_scopeConfig = $scopeConfig;
        $this->messageManager = $messageManager;
    }

    protected function loadFilter() {
        $email_list = preg_split('/,|' . "\n" . '/', $this->_scopeConfig->getValue(self::EMAIL_BLOCKER_ADDRESSES_CONFIG_PATH, \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE));
        $validated_list = array();
        foreach ($email_list as $address) {
            $address = trim($address);
            if ($address != '') {
                $validated_list[] = $address;
            }
        }
        return $validated_list;
    }

    protected function loadFilterCustomerAddresses() {
        $address_list = preg_split('/,|' . "\n" . '/', $this->_scopeConfig->getValue(self::CUSTOMER_BLOCKER_ADDRESSES_CONFIG_PATH, \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE));
        $validated_list = array();
        foreach ($address_list as $address) {
            $address = trim($address);
            if ($address != '') {
                $validated_list[] = $address;
            }
        }
        return $validated_list;
    }

    protected function checkCustomerAddress(\Magento\Framework\Event\Observer $observer, $isPayPalOrder = false)
    {
        $addressToBlock = false;
        // Get address list to block
        $customer_address = $this->loadFilterCustomerAddresses();

        if(!empty($customer_address))
        {
            // Get order data based on payment method
            if($isPayPalOrder)
            {
                $orderData = $observer->getEvent()->getQuote()->getShippingAddress()->getData();
            }
            else
            {
                $orderData = $observer->getEvent()->getOrder()->getShippingAddress()->getData();
            }

            $orderShippingAddressData = array_map('strtolower', $orderData);

            // Cleaning of the Street field ot the order with special characters
            $orderStreet = preg_replace('/[^0-9a-zA-Z\s]/', '', $orderShippingAddressData['street']);
            $arrayOrderStreet = explode(" ", $orderStreet);
            foreach ($customer_address as $address)
            {
                $check = false;
                $arrayAddress = array_map('strtolower', explode("|", $address));
                $arrayAddress = array_map('trim', $arrayAddress);
                $arrayStreet = [];

                // I check if the last value is the address
                if(strpos($arrayAddress[count($arrayAddress)-1], '+') !== false)
                {
                    // Array with the data of the street to be blocked
                    $arrayStreet = array_map('trim', explode("+", strtolower(array_pop($arrayAddress))));
                    $arrayStreet = array_filter($arrayStreet);
                }

                // If the street value is not set, I check the other elements of the address
                if(empty($arrayStreet))
                {
                    $check = true;
                }
                else
                {
                    // Check if the street set coincides with the street of order
                    $checkStreetValue = array_intersect($arrayStreet, $arrayOrderStreet);
                    if(count($checkStreetValue) >= count($arrayStreet))
                    {
                        $check = true;
                    }
                }

                if($check)
                {
                    // Check the other values ​​set for the address to be blocked
                    foreach ($arrayAddress as $val)
                    {
                        $check = $check && in_array($val, $orderShippingAddressData);
                    }
                }


                // If all the set values​are present in the shipping address of the order, I block it
                if($check)
                {
                    $addressToBlock = true;
                    break;
                }
            }
        }

        return $addressToBlock;
    }

    public function execute(\Magento\Framework\Event\Observer $observer) {

    }

}
